function _asyncIterator(iterable) { if (typeof Symbol === "function") { if (Symbol.asyncIterator) { var method = iterable[Symbol.asyncIterator]; if (method != null) return method.call(iterable); } if (Symbol.iterator) { return iterable[Symbol.iterator](); } } throw new TypeError("Object is not async iterable"); }

function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume("next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _AwaitValue(value) { this.wrapped = value; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const defaultConfig = require('./config');

const dynamodb = require('./dynamodb');

const s3 = require('./s3');

const {
  createHash
} = require('crypto');

const {
  resolveConfig,
  retryPromiseWithJitteredBackoff,
  parseKeyProperties,
  dateString
} = require('./utils');

const {
  ConfigurationError,
  BusyCommittingError,
  AggregateNotFoundError
} = require('./errors');

class Aggregate {
  static reducer() {
    throw new Error('You must specify a reducer: (state, event, commit) => newState');
  }

  static stateToJS(state) {
    return state;
  }

  static jsToState(js) {
    return js;
  }

  static create(props = {}, constructorParams) {
    var _this = this;

    return _asyncToGenerator(function* () {
      const {
        keyString: aggregateKey,
        keyProps
      } = parseKeyProperties({
        schema: _this.keySchema,
        props,
        separator: _this.keySchemaSeparator
      });
      const instance = new _this(_extends({
        aggregateKey
      }, constructorParams));
      yield instance.hydrate();
      yield instance.create(_extends({}, props, keyProps));
      return instance;
    })();
  }

  static loadOrCreate(...args) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      return (yield _this2.load(...args)) || (yield _this2.create(...args));
    })();
  }

  static getState(...args) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const instance = yield _this3.load(...args);
      return instance && instance.state;
    })();
  }

  static keyFromProps(props) {
    const {
      keyString
    } = parseKeyProperties({
      schema: this.keySchema,
      props,
      separator: this.keySchemaSeparator
    });
    return keyString;
  }

  static load(a, b) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      const aggregateKey = typeof a === 'string' ? a : _this4.keyFromProps(a);
      const props = b || a;

      const _ref = props || {},
            {
        version,
        time,
        fail,
        consistentRead
      } = _ref,
            rest = _objectWithoutProperties(_ref, ["version", "time", "fail", "consistentRead"]);

      const aggregate = new _this4(_extends({}, rest, {
        aggregateKey
      }));
      yield aggregate.hydrate({
        version,
        time,
        consistentRead
      });

      if (_this4.keySchema && !(aggregate.version && Object.keys(aggregate.state).length)) {
        if (fail) {
          throw new AggregateNotFoundError(`${aggregate.aggregateType} with key '${aggregateKey}' does exist`);
        }

        return null;
      }

      return aggregate;
    })();
  }

  static commit(...args) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      let keyProps, events;

      if (args.length === 1) {
        keyProps = {};
        events = args[0];
      } else {
        keyProps = args[0];
        events = args[1];
      }

      const commitStartedAt = new Date();
      const aggregateType = _this5.name;

      const aggregateKey = _this5.keyFromProps(keyProps);

      return yield retryPromiseWithJitteredBackoff(
      /*#__PURE__*/
      function () {
        var _ref2 = _asyncToGenerator(function* (attempts) {
          const headCommit = yield dynamodb.getAggregateHeadCommit({
            aggregateType,
            aggregateKey
          });
          const commitData = {
            aggregateType,
            aggregateKey,
            version: headCommit ? headCommit.version + 1 : 1,
            events
          };
          const commitRecord = dynamodb.serializeCommit(commitData);
          yield dynamodb.commit(commitRecord);
          const totalTimeSpent = new Date() - commitStartedAt;

          _this5.logger.debug({
            aggregateType,
            aggregateKey,
            function: 'commit',
            totalTimeSpent,
            attempts
          }, `${_this5.aggregateType}:${_this5.aggregateKey} commit() ${events.map(event => event.type).join(', ')} (${totalTimeSpent}ms)`);

          return dynamodb.deserializeCommit(commitRecord);
        });

        return function (_x) {
          return _ref2.apply(this, arguments);
        };
      }(), {
        initialDelay: 5,
        maxAttempts: 10,
        shouldRetry: error => error.code === 'ConditionalCheckFailedException'
      });
    })();
  }

  static scanInstances(_ref3 = {}) {
    var _this6 = this;

    return _wrapAsyncGenerator(function* () {
      let {
        transformConcurrency
      } = _ref3,
          rest = _objectWithoutProperties(_ref3, ["transformConcurrency"]);

      let currentAggregate = null;
      const commits = dynamodb.queryCommits(_extends({
        aggregateType: _this6.name,
        upcasters: _this6.upcasters,
        transform: _this6.lazyTransformation,
        transformConcurrency
      }, rest));
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;

      var _iteratorError;

      try {
        for (var _iterator = _asyncIterator(commits), _step, _value; _step = yield _awaitAsyncGenerator(_iterator.next()), _iteratorNormalCompletion = _step.done, _value = yield _awaitAsyncGenerator(_step.value), !_iteratorNormalCompletion; _iteratorNormalCompletion = true) {
          const commit = _value;
          const thisAggregateKey = commit.aggregateKey;

          if (!currentAggregate || currentAggregate.aggregateKey !== thisAggregateKey) {
            if (currentAggregate) yield currentAggregate;
            currentAggregate = new _this6({
              aggregateKey: thisAggregateKey
            });
          }

          currentAggregate.processCommit(commit);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            yield _awaitAsyncGenerator(_iterator.return());
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      if (currentAggregate) {
        yield currentAggregate;
      }
    })();
  }

  static get instances() {
    return this.scanInstances();
  }

  static get upcastersChecksum() {
    return Object.keys(this.upcasters).length ? createHash('md5').update(JSON.stringify(Object.keys(this.upcasters).map(eventType => ({
      eventType,
      versions: Object.keys(this.upcasters[eventType])
    })))).digest('base64') : undefined;
  }

  constructor(_ref4 = {}) {
    let {
      aggregateType,
      aggregateKey = '0'
    } = _ref4,
        constructorConfig = _objectWithoutProperties(_ref4, ["aggregateType", "aggregateKey"]);

    this.config = resolveConfig([defaultConfig, this.constructor, constructorConfig], ['snapshots', 'snapshotsBucket', 'snapshotsFrequency', 'snapshotsPrefix', 'tableName', 'logger']);
    this.version = 0;
    this.logger = this.config.logger;

    if (typeof this.config.tableName !== 'string' || !this.config.tableName.match(/^[a-zA-Z0-9\_\-\.]{3,255}$/)) {
      throw new ConfigurationError('You need to specify a valid table name');
    }

    if (this.config.snapshots) {
      if (!this.config.snapshotsFrequency) {
        throw new Error('When snapshots are enabled, you need to specify snapshotsFrequency');
      } else if (!this.config.snapshotsBucket) {
        throw new Error('When snapshots are enabled, you need to specify snapshotsBucket');
      }
    }

    this.aggregateType = aggregateType || this.constructor.name;
    this.aggregateKey = aggregateKey;
  } // Getter for state to allow for non-plain js state types


  get state() {
    return this.constructor.stateToJS(this._state || this.constructor.jsToState({}));
  }

  create() {
    throw new Error(`${this.constructor.name} class does not have instance method create()`);
  } // Provide a cleaned up version of the instance for debugging


  inspect() {
    const {
      aggregateType,
      aggregateKey,
      version,
      state
    } = this;
    return {
      aggregateType,
      aggregateKey,
      version,
      state
    };
  }

  commit(eventOrEvents, {
    retry = false
  } = {}) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      const commitStartedAt = new Date();
      let writeUnitsConsumed = 0;
      const events = Array.isArray(eventOrEvents) ? eventOrEvents : [eventOrEvents];

      if (_this7.commitInFlight) {
        throw new BusyCommittingError(`Busy committing version ${_this7.commitInFlight.version}`);
      }

      const perform =
      /*#__PURE__*/
      function () {
        var _ref5 = _asyncToGenerator(function* () {
          try {
            const commitData = {
              aggregateType: _this7.aggregateType,
              aggregateKey: _this7.aggregateKey,
              version: _this7.version + 1,
              committedAt: new Date(),
              events
            };
            _this7.commitInFlight = commitData;
            const commitRecord = dynamodb.serializeCommit(commitData);
            const commitResult = yield dynamodb.commit(commitRecord);
            const {
              ConsumedCapacity
            } = commitResult;

            if (ConsumedCapacity) {
              writeUnitsConsumed += ConsumedCapacity.CapacityUnits;
            }

            _this7.processCommit(dynamodb.deserializeCommit(commitRecord));

            if (_this7.config.snapshots && _this7.version % _this7.config.snapshotsFrequency === 0) {
              yield _this7.writeSnapshot();
            }
          } finally {
            _this7.commitInFlight = null;
          }
        });

        return function perform() {
          return _ref5.apply(this, arguments);
        };
      }();

      if (retry) {
        return retryPromiseWithJitteredBackoff(
        /*#__PURE__*/
        function () {
          var _ref6 = _asyncToGenerator(function* (attempts) {
            yield perform();
            const totalTimeSpent = new Date() - commitStartedAt;

            _this7.config.logger.debug({
              aggregateType: _this7.aggregateType,
              aggregateKey: _this7.aggregateKey,
              function: 'commit',
              writeUnitsConsumed,
              totalTimeSpent,
              attempts
            }, `${_this7.aggregateType}:${_this7.aggregateKey} commit() ${events.map(event => event.type).join(', ')} (${totalTimeSpent}ms)`);

            return _this7;
          });

          return function (_x2) {
            return _ref6.apply(this, arguments);
          };
        }(), _extends({
          initialDelay: 5,
          maxAttempts: 10,
          shouldRetry: error => error.code === 'ConditionalCheckFailedException',
          beforeRetry: () => _this7.hydrate()
        }, typeof retry === 'object' && retry));
      }

      yield perform();
      const totalTimeSpent = new Date() - commitStartedAt;

      _this7.logger.debug({
        aggregateType: _this7.aggregateType,
        aggregateKey: _this7.aggregateKey,
        function: 'commit',
        totalTimeSpent,
        writeUnitsConsumed,
        eventTypes: events.map(event => event.type)
      }, `${_this7.aggregateType}:${_this7.aggregateKey} commit() ${events.map(event => event.type).join(', ')} (${totalTimeSpent}ms)`);

      return _this7;
    })();
  }

  hydrate({
    version: requestedVersion,
    time: requestedTime,
    consistentRead = true
  } = {}) {
    var _this8 = this;

    return _asyncToGenerator(function* () {
      const hydrateStartedAt = new Date();
      const requestedTimeString = dateString(requestedTime);

      if (requestedVersion < _this8.version || requestedTimeString < _this8.headCommitTimestamp) {
        _this8.reset();
      }

      let timeSpentQuerying = 0;
      let timeSpentProcessing = 0;
      let snapshotVersionLoaded = null;
      let timeSpentReadingSnapshot = 0;
      let readUnitsConsumed = 0;
      let timeSpentRewritingSnapshot = 0;
      const hydratedFromVersion = _this8.version;
      let shouldRewriteSnapshot = false;

      if (_this8.config.snapshots) {
        const snapshotReadStart = new Date();
        const snapshot = yield s3.readAggregateSnapshot({
          aggregateType: _this8.aggregateType,
          aggregateKey: _this8.aggregateKey
        }, _this8.config);
        timeSpentReadingSnapshot = new Date() - snapshotReadStart;

        if (snapshot) {
          let shouldUseSnapshot = false;
          const {
            state,
            version,
            upcastersChecksum,
            headCommitTimestamp
          } = snapshot;

          if (upcastersChecksum !== _this8.constructor.upcastersChecksum) {
            shouldRewriteSnapshot = true;
          } else if (requestedVersion) {
            shouldUseSnapshot = requestedVersion >= version;
          } else if (requestedTime) {
            shouldUseSnapshot = requestedTimeString >= headCommitTimestamp;
          } else if (version > _this8.version) {
            shouldUseSnapshot = true;
          }

          if (shouldUseSnapshot) {
            _this8._state = _this8.constructor.jsToState(state);
            _this8.version = version;
            snapshotVersionLoaded = version;
          }
        }
      } // not necessarily requested time or version


      if (_this8.version !== requestedVersion) {
        const commits = dynamodb.getAggregateCommits({
          aggregateType: _this8.aggregateType,
          aggregateKey: _this8.aggregateKey,
          maxVersion: requestedVersion,
          minVersion: _this8.version + 1,
          maxTime: requestedTime,
          upcasters: _this8.constructor.upcasters,
          transform: _this8.constructor.lazyTransformation,
          consistentRead
        }, ({
          consumedCapacityUnits,
          queryTime
        }) => {
          readUnitsConsumed += consumedCapacityUnits;
          timeSpentQuerying += queryTime;
        });
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;

        var _iteratorError2;

        try {
          for (var _iterator2 = _asyncIterator(commits), _step2, _value2; _step2 = yield _iterator2.next(), _iteratorNormalCompletion2 = _step2.done, _value2 = yield _step2.value, !_iteratorNormalCompletion2; _iteratorNormalCompletion2 = true) {
            const commit = _value2;
            const processingStartedAt = new Date();

            _this8.processCommit(commit);

            timeSpentProcessing += new Date() - processingStartedAt;
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
              yield _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }

      if (shouldRewriteSnapshot) {
        const rewriteSnapshotStartedAt = new Date();
        yield _this8.writeSnapshot();
        timeSpentRewritingSnapshot = new Date() - rewriteSnapshotStartedAt;
      }

      const totalTimeSpent = new Date() - hydrateStartedAt;

      _this8.logger.debug({
        timeSpentReadingSnapshot,
        timeSpentQuerying,
        timeSpentProcessing,
        timeSpentRewritingSnapshot,
        aggregateType: _this8.aggregateType,
        aggregateKey: _this8.aggregateKey,
        snapshotVersionLoaded,
        totalTimeSpent,
        hydratedFromVersion,
        readUnitsConsumed,
        version: _this8.version,
        function: 'hydrate'
      }, _this8.version > hydratedFromVersion ? `${_this8.aggregateType}:${_this8.aggregateKey} hydrate() version ${hydratedFromVersion} => ${_this8.version} ${snapshotVersionLoaded ? `[loaded version ${snapshotVersionLoaded} from snapshot] ` : ''}(${totalTimeSpent}ms)` : `${_this8.aggregateType}:${_this8.aggregateKey} hydrate() no new commits, still at version ${_this8.version}`);

      return _this8;
    })();
  }

  processCommit(commit) {
    if (!commit) throw new Error('Commit not provided to processCommit()');
    this._state = commit.events.reduce((state, event) => this.constructor.reducer(state, event, commit), this._state);
    this.version = commit.version;
    this.headCommitTimestamp = commit.committedAt;
    this.headCommitId = commit.commitId;
  }

  writeSnapshot() {
    var _this9 = this;

    return _asyncToGenerator(function* () {
      const {
        version,
        state,
        aggregateType,
        aggregateKey,
        headCommitTimestamp
      } = _this9;
      yield s3.writeAggregateSnapshot({
        aggregateType,
        aggregateKey,
        version,
        state,
        headCommitTimestamp,
        upcastersChecksum: _this9.constructor.upcastersChecksum
      });
    })();
  }

  reset() {
    this.version = 0;
    this._state = undefined;
    this.headCommitId = undefined;
    this.headCommitTimestamp = undefined;
  }

  get aggregateId() {
    return [this.aggregateType, this.aggregateKey].join(':');
  }

}

Object.assign(Aggregate, {
  upcasters: {},
  lazyTransformation: false,
  keySchemaSeparator: '.',
  logger: defaultConfig.logger
});
module.exports = Aggregate;