function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const config = require('../config');

const tableSchema = require('./tableSchema');

function createTable() {
  return _createTable.apply(this, arguments);
}

function _createTable() {
  _createTable = _asyncToGenerator(function* ({
    tableName = config.tableName,
    wait = true,
    checkStatusIntervalInMs = 500,
    timeoutInSeconds = 120,
    tableWriteCapacity = 1,
    tableReadCapacity = tableWriteCapacity,
    indexWriteCapacity = tableWriteCapacity,
    indexReadCapacity = tableReadCapacity,
    logger = config.logger
  } = {}) {
    const AWS = config.configuredAWS;
    const ddb = new AWS.DynamoDB();

    if (!tableName) {
      throw new Error('You must provide a tableName');
    }

    try {
      logger.info(`Creating table '${tableName}'`);
      yield ddb.createTable(_extends({
        TableName: tableName
      }, tableSchema({
        tableWriteCapacity,
        tableReadCapacity,
        indexWriteCapacity,
        indexReadCapacity
      }))).promise();
    } catch (error) {
      if (error.code === 'ResourceInUseException') {
        return {
          tableWasCreated: false
        };
      }

      throw error;
    }

    if (!wait) return {
      tableWasCreated: true
    };
    let timedOut = false;
    const timer = setTimeout(() => {
      timedOut = true;
    }, timeoutInSeconds * 1000);

    while (!timedOut) {
      const {
        Table: {
          TableStatus
        }
      } = yield ddb.describeTable({
        TableName: tableName
      }).promise();

      switch (TableStatus) {
        case 'ACTIVE':
          clearTimeout(timer);
          return {
            tableWasCreated: true
          };

        case 'CREATING':
          yield new Promise(resolve => setTimeout(resolve, checkStatusIntervalInMs));
          continue;

        default:
          {
            throw new Error('Invalid status ${TableStatus} while waiting for table to be created');
          }
      }
    }

    throw new Error('Table did not become active within the given time');
  });
  return _createTable.apply(this, arguments);
}

module.exports = createTable;