function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const config = require('../config');

function deleteTable() {
  return _deleteTable.apply(this, arguments);
}

function _deleteTable() {
  _deleteTable = _asyncToGenerator(function* ({
    tableName = config.tableName,
    wait = true,
    checkStatusIntervalInMs = 500,
    timeoutInSeconds = 60,
    logger = config.logger
  } = {}) {
    const AWS = config.configuredAWS;
    const ddb = new AWS.DynamoDB();

    if (!tableName) {
      throw new Error('You must provide a tableName');
    }

    let timer;

    try {
      logger.info(`Destroying table '${tableName}'`);
      yield ddb.deleteTable({
        TableName: tableName
      }).promise();
      if (!wait) return {
        tableWasDestroyed: true
      };
      let timedOut = false;
      timer = setTimeout(() => {
        timedOut = true;
      }, timeoutInSeconds * 1000);

      while (!timedOut) {
        const {
          Table: {
            TableStatus
          }
        } = yield ddb.describeTable({
          TableName: tableName
        }).promise();

        switch (TableStatus) {
          case 'DELETING':
            yield new Promise(resolve => setTimeout(resolve, checkStatusIntervalInMs));
            continue;

          default:
            {
              throw new Error('Invalid status ${TableStatus} while waiting for table to be deleteTableed');
            }
        }
      }

      throw new Error('Table was not destroyed in time');
    } catch (error) {
      if (error.code === 'ResourceNotFoundException') {
        clearTimeout(timer);
        return {
          tableWasDestroyed: true
        };
      }

      throw error;
    }
  });
  return _deleteTable.apply(this, arguments);
}

module.exports = deleteTable;