function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

function _awaitAsyncGenerator(value) { return new _AwaitValue(value); }

function _wrapAsyncGenerator(fn) { return function () { return new _AsyncGenerator(fn.apply(this, arguments)); }; }

function _AsyncGenerator(gen) { var front, back; function send(key, arg) { return new Promise(function (resolve, reject) { var request = { key: key, arg: arg, resolve: resolve, reject: reject, next: null }; if (back) { back = back.next = request; } else { front = back = request; resume(key, arg); } }); } function resume(key, arg) { try { var result = gen[key](arg); var value = result.value; var wrappedAwait = value instanceof _AwaitValue; Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) { if (wrappedAwait) { resume("next", arg); return; } settle(result.done ? "return" : "normal", arg); }, function (err) { resume("throw", err); }); } catch (err) { settle("throw", err); } } function settle(type, value) { switch (type) { case "return": front.resolve({ value: value, done: true }); break; case "throw": front.reject(value); break; default: front.resolve({ value: value, done: false }); break; } front = front.next; if (front) { resume(front.key, front.arg); } else { back = null; } } this._invoke = send; if (typeof gen.return !== "function") { this.return = undefined; } }

if (typeof Symbol === "function" && Symbol.asyncIterator) { _AsyncGenerator.prototype[Symbol.asyncIterator] = function () { return this; }; }

_AsyncGenerator.prototype.next = function (arg) { return this._invoke("next", arg); };

_AsyncGenerator.prototype.throw = function (arg) { return this._invoke("throw", arg); };

_AsyncGenerator.prototype.return = function (arg) { return this._invoke("return", arg); };

function _AwaitValue(value) { this.wrapped = value; }

const config = require('../config');

const deserializeCommit = require('./deserializeCommit');

const serializeCommit = require('./serializeCommit');

const upcastCommit = require('../utils/upcastCommit');

const {
  asyncify,
  eachLimit
} = require('async');

function queryCommits() {
  return _queryCommits.apply(this, arguments);
}

function _queryCommits() {
  _queryCommits = _wrapAsyncGenerator(function* ({
    aggregateType,
    consistentRead = true,
    upcasters = {},
    transform = false,
    queryExpression,
    queryVariables = {},
    filterExpression,
    filterVariables = {},
    transformConcurrency = 10,
    tableName = config.tableName,
    limit,
    descending = false
  } = {}, reportCallback) {
    const AWS = config.configuredAWS;
    const ddb = new AWS.DynamoDB();
    let queryTime = 0;
    let consumedCapacityUnits = 0;
    let queryResult = {};

    const query =
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(function* () {
        const {
          LastEvaluatedKey: ExclusiveStartKey
        } = queryResult;
        const queryStartedAt = new Date();
        queryResult = yield ddb.query(_extends({
          TableName: tableName
        }, ExclusiveStartKey && {
          ExclusiveStartKey
        }, {
          ConsistentRead: consistentRead,
          KeyConditionExpression: `a = :aggregateType ${queryExpression ? `AND ${queryExpression}` : ''}`
        }, filterExpression && {
          FilterExpression: filterExpression
        }, {
          ExpressionAttributeValues: AWS.DynamoDB.Converter.marshall(_extends({
            ':aggregateType': aggregateType
          }, queryVariables, filterVariables))
        }, limit && {
          Limit: limit
        }, descending && {
          ScanIndexForward: false
        })).promise();
        queryTime += new Date() - queryStartedAt;
        const {
          ConsumedCapacity
        } = queryResult;
        consumedCapacityUnits += ConsumedCapacity ? ConsumedCapacity.CapacityUnits : 1;
        return queryResult;
      });

      return function query() {
        return _ref.apply(this, arguments);
      };
    }();

    yield _awaitAsyncGenerator(query());

    do {
      let commits = queryResult.Items.map(deserializeCommit);

      if (upcasters) {
        commits = commits.map(commit => upcastCommit(commit, upcasters));
      }

      let transformPromise;

      if (transform) {
        transformPromise = new Promise((resolve, reject) => {
          eachLimit(commits.filter(commit => commit.upcasted), transformConcurrency, asyncify(commit => ddb.putItem({
            TableName: tableName,
            Item: serializeCommit(commit),
            ReturnValues: 'NONE'
          }).promise()), err => err ? reject(err) : resolve());
        });
      }

      for (const commit of commits) {
        yield commit;
      }

      yield _awaitAsyncGenerator(Promise.all([transformPromise, query()]));
    } while (queryResult.LastEvaluatedKey);

    if (reportCallback) {
      reportCallback({
        consumedCapacityUnits,
        queryTime
      });
    }
  });
  return _queryCommits.apply(this, arguments);
}

module.exports = queryCommits;