function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const config = require('../config');

function removeAutoScaling() {
  return _removeAutoScaling.apply(this, arguments);
}

function _removeAutoScaling() {
  _removeAutoScaling = _asyncToGenerator(function* ({
    tableName = config.tableName
  } = {}) {
    const RoleName = `${tableName}_DDBAutoScalingRole`;
    const AWS = config.configuredAWS;
    const iam = new AWS.IAM();
    const autoscaling = new AWS.ApplicationAutoScaling();
    const autoScalingTargets = [{
      ResourceId: `table/${tableName}`,
      type: 'table'
    }, {
      ResourceId: `table/${tableName}/index/commitIdIndex`,
      type: 'index'
    }];

    for (const _ref of autoScalingTargets) {
      const {
        ResourceId,
        type
      } = _ref;

      try {
        yield autoscaling.deregisterScalableTarget({
          ResourceId,
          ServiceNamespace: 'dynamodb',
          ScalableDimension: `dynamodb:${type}:ReadCapacityUnits`
        }).promise();
        yield autoscaling.deregisterScalableTarget({
          ResourceId,
          ServiceNamespace: 'dynamodb',
          ScalableDimension: `dynamodb:${type}:WriteCapacityUnits`
        }).promise();
      } catch (error) {
        if (error.code !== 'ObjectNotFoundException') {
          throw error;
        }
      }
    }

    try {
      yield iam.deleteRolePolicy({
        PolicyName: 'default',
        RoleName
      }).promise();
      yield iam.deleteRole({
        RoleName
      }).promise();
    } catch (error) {
      if (!['ObjectNotFoundException', 'NoSuchEntity'].includes(error.code)) {
        throw error;
      }
    }
  });
  return _removeAutoScaling.apply(this, arguments);
}

module.exports = removeAutoScaling;