function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const config = require('../config');

function setupAutoScaling() {
  return _setupAutoScaling.apply(this, arguments);
}

function _setupAutoScaling() {
  _setupAutoScaling = _asyncToGenerator(function* ({
    tableName = config.tableName,
    tableReadMinCapacity = 1,
    tableReadMaxCapacity = 2,
    tableWriteMinCapacity = 1,
    tableWriteMaxCapacity = 2,
    tableScaleInCooldown = 60,
    tableScaleOutCooldown = 60,
    indexReadMinCapacity = tableReadMinCapacity,
    indexReadMaxCapacity = tableReadMaxCapacity,
    indexWriteMinCapacity = tableWriteMinCapacity,
    indexWriteMaxCapacity = tableWriteMaxCapacity,
    indexScaleInCooldown = tableScaleInCooldown,
    indexScaleOutCooldown = tableScaleOutCooldown,
    utilizationTargetInPercent = 70
  } = {}) {
    const RoleName = `${tableName}_DDBAutoScalingRole`;
    const AWS = config.configuredAWS;
    const iam = new AWS.IAM();
    const autoscaling = new AWS.ApplicationAutoScaling();
    let role;
    const AssumeRolePolicyDocument = {
      Version: '2012-10-17',
      Statement: [{
        Effect: 'Allow',
        Principal: {
          Service: 'application-autoscaling.amazonaws.com'
        },
        Action: 'sts:AssumeRole'
      }]
    };
    const inlineRolePolicy = {
      Version: '2012-10-17',
      Statement: [{
        Effect: 'Allow',
        Action: ['dynamodb:DescribeTable', 'dynamodb:UpdateTable', 'cloudwatch:PutMetricAlarm', 'cloudwatch:DescribeAlarms', 'cloudwatch:DeleteAlarms'],
        Resource: '*'
      }]
    };

    try {
      role = yield iam.createRole({
        AssumeRolePolicyDocument: JSON.stringify(AssumeRolePolicyDocument),
        RoleName
      }).promise();
    } catch (error) {
      if (error.code !== 'EntityAlreadyExists') {
        throw error;
      }

      yield iam.updateAssumeRolePolicy({
        RoleName,
        PolicyDocument: JSON.stringify(AssumeRolePolicyDocument)
      }).promise();
      role = yield iam.getRole({
        RoleName
      }).promise();
    }

    yield iam.putRolePolicy({
      RoleName,
      PolicyName: 'default',
      PolicyDocument: JSON.stringify(inlineRolePolicy)
    }).promise();
    const autoScalingTargets = [{
      ResourceId: `table/${tableName}`,
      ScaleInCooldown: tableScaleInCooldown,
      ScaleOutCooldown: tableScaleOutCooldown,
      type: 'table',
      readMin: tableReadMinCapacity,
      readMax: tableReadMaxCapacity,
      writeMin: tableWriteMinCapacity,
      writeMax: tableWriteMaxCapacity
    }, {
      ResourceId: `table/${tableName}/index/commitIdIndex`,
      ScaleInCooldown: indexScaleInCooldown,
      ScaleOutCooldown: indexScaleOutCooldown,
      type: 'index',
      readMin: indexReadMinCapacity,
      readMax: indexReadMaxCapacity,
      writeMin: indexWriteMinCapacity,
      writeMax: indexWriteMaxCapacity
    }];

    const registerTargets =
    /*#__PURE__*/
    function () {
      var _ref = _asyncToGenerator(function* () {
        for (const _ref2 of autoScalingTargets) {
          const {
            ResourceId,
            readMin,
            readMax,
            writeMin,
            writeMax,
            type,
            ScaleInCooldown,
            ScaleOutCooldown
          } = _ref2;
          yield autoscaling.registerScalableTarget({
            ServiceNamespace: 'dynamodb',
            ResourceId,
            ScalableDimension: `dynamodb:${type}:ReadCapacityUnits`,
            MinCapacity: readMin,
            MaxCapacity: readMax,
            RoleARN: role.Role.Arn
          }).promise();
          yield autoscaling.registerScalableTarget({
            ServiceNamespace: 'dynamodb',
            ResourceId,
            ScalableDimension: `dynamodb:${type}:WriteCapacityUnits`,
            MinCapacity: writeMin,
            MaxCapacity: writeMax,
            RoleARN: role.Role.Arn
          }).promise();
          yield autoscaling.putScalingPolicy({
            ResourceId,
            ServiceNamespace: 'dynamodb',
            ScalableDimension: `dynamodb:${type}:ReadCapacityUnits`,
            PolicyName: `DynamoDBReadCapacityUtilization:${ResourceId}`,
            PolicyType: 'TargetTrackingScaling',
            TargetTrackingScalingPolicyConfiguration: {
              PredefinedMetricSpecification: {
                PredefinedMetricType: 'DynamoDBReadCapacityUtilization'
              },
              ScaleOutCooldown,
              ScaleInCooldown,
              TargetValue: utilizationTargetInPercent
            }
          }).promise();
          yield autoscaling.putScalingPolicy({
            ResourceId,
            ServiceNamespace: 'dynamodb',
            ScalableDimension: `dynamodb:${type}:WriteCapacityUnits`,
            PolicyName: `DynamoDBWriteCapacityUtilization:${ResourceId}`,
            PolicyType: 'TargetTrackingScaling',
            TargetTrackingScalingPolicyConfiguration: {
              PredefinedMetricSpecification: {
                PredefinedMetricType: 'DynamoDBWriteCapacityUtilization'
              },
              ScaleOutCooldown,
              ScaleInCooldown,
              TargetValue: utilizationTargetInPercent
            }
          }).promise();
        }
      });

      return function registerTargets() {
        return _ref.apply(this, arguments);
      };
    }();

    for (let attempt = 0; attempt < 20; attempt++) {
      try {
        yield registerTargets();
        return;
      } catch (error) {
        if (error.code !== 'ValidationException' || !(error.message.startsWith('Unable to assume IAM role') || error.message.includes('Reason: The security token included in the request is invalid.'))) {
          throw error;
        }

        yield new Promise(resolve => setTimeout(resolve, 500));
      }
    }

    throw new Error('Exhausted attempts to create scaling policies');
  });
  return _setupAutoScaling.apply(this, arguments);
}

module.exports = setupAutoScaling;