function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const {
  retryPromiseWithJitteredBackoff
} = require('./utils');

const config = require('./config');

function aggregateCommand(func, {
  retry
} = {}) {
  return (
    /*#__PURE__*/
    function () {
      var _decoratedCommand = _asyncToGenerator(function* (...args) {
        const aggregate = this; // eslint-disable-line

        const invocationStartedAt = new Date();
        let timeSpentExecutingCommand = 0;

        function runWithValidaton(_x) {
          return _runWithValidaton.apply(this, arguments);
        }

        function _runWithValidaton() {
          _runWithValidaton = _asyncToGenerator(function* (attempt) {
            const commandExecutionStartedAt = new Date();
            let result;

            try {
              result = yield func.apply(aggregate, args);
            } finally {
              timeSpentExecutingCommand += new Date() - commandExecutionStartedAt;
            }

            const totalTimeSpent = new Date() - invocationStartedAt;
            config.logger.debug({
              totalTimeSpent,
              attempts: attempt,
              timeSpentExecutingCommand,
              aggregateId: aggregate.aggregateId,
              command: func.name
            }, `${aggregate.aggregateId} ${func.name}() completed (${totalTimeSpent}ms)`);
            return result;
          });
          return _runWithValidaton.apply(this, arguments);
        }

        if (retry) {
          return retryPromiseWithJitteredBackoff(runWithValidaton, _extends({
            initialDelay: 5,
            maxAttempts: 10,
            shouldRetry: error => error.code === 'ConditionalCheckFailedException',
            beforeRetry: () => aggregate.hydrate()
          }, typeof retry === 'object' && retry));
        }

        return yield runWithValidaton();
      });

      return function decoratedCommand() {
        return _decoratedCommand.apply(this, arguments);
      };
    }()
  );
}

module.exports = aggregateCommand;