function _asyncIterator(iterable) { if (typeof Symbol === "function") { if (Symbol.asyncIterator) { var method = iterable[Symbol.asyncIterator]; if (method != null) return method.call(iterable); } if (Symbol.iterator) { return iterable[Symbol.iterator](); } } throw new TypeError("Object is not async iterable"); }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } } function _next(value) { step("next", value); } function _throw(err) { step("throw", err); } _next(); }); }; }

const config = require('../config');

const serializeCommit = require('./serializeCommit');

class BatchMutator {
  constructor({
    tableName = config.tableName,
    AWS = config.configuredAWS,
    maxItemsPerRequest = 25,
    queueSize = maxItemsPerRequest
  } = {}) {
    Object.assign(this, {
      queueSize,
      maxItemsPerRequest,
      AWS,
      tableName,
      queue: [],
      pendingAddToQueuePromises: [],
      drained: Promise.resolve()
    });
  }

  asIterable(something) {
    return Array.isArray(something) || something.next ? something : [something];
  }

  delete(commits) {
    var _this = this;

    return _asyncToGenerator(function* () {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;

      var _iteratorError;

      try {
        for (var _iterator = _asyncIterator(_this.asIterable(commits)), _step, _value; _step = yield _iterator.next(), _iteratorNormalCompletion = _step.done, _value = yield _step.value, !_iteratorNormalCompletion; _iteratorNormalCompletion = true) {
          const commit = _value;
          const {
            a,
            k
          } = serializeCommit(commit);
          yield _this.addToQueue({
            DeleteRequest: {
              Key: {
                a,
                k
              }
            }
          });
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            yield _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    })();
  }

  put(commits) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;

      var _iteratorError2;

      try {
        for (var _iterator2 = _asyncIterator(_this2.asIterable(commits)), _step2, _value2; _step2 = yield _iterator2.next(), _iteratorNormalCompletion2 = _step2.done, _value2 = yield _step2.value, !_iteratorNormalCompletion2; _iteratorNormalCompletion2 = true) {
          const commit = _value2;
          yield _this2.addToQueue({
            PutRequest: {
              Item: serializeCommit(commit)
            }
          });
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            yield _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    })();
  }

  addToQueue(item) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      if (_this3.queue.length >= _this3.queueSize) {
        yield new Promise(resolve => _this3.pendingAddToQueuePromises.push(resolve));
      }

      _this3.queue.push(item);

      if (!_this3.workerLoopRunning) {
        _this3.workerLoop();
      }
    })();
  }

  workerLoop() {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      let resolveFn;
      _this4.drained = new Promise(resolve => resolveFn = resolve);
      _this4.workerLoopRunning = true;

      while (_this4.queue.length) {
        const items = _this4.queue.splice(0, _this4.maxItemsPerRequest);

        while (_this4.queue.length <= _this4.queueSize && _this4.pendingAddToQueuePromises.length) {
          _this4.pendingAddToQueuePromises.shift()();
        }

        const ddb = new _this4.AWS.DynamoDB();
        const {
          UnprocessedItems
        } = yield ddb.batchWriteItem({
          RequestItems: {
            [_this4.tableName]: items
          }
        }).promise();
        const itemsToRequeue = UnprocessedItems[_this4.tableName];

        if (itemsToRequeue) {
          _this4.queue.splice(0, 0, ...itemsToRequeue);
        }
      }

      _this4.workerLoopRunning = false;
      resolveFn();
    })();
  }

}

module.exports = BatchMutator;